;*************************************************************************
; Copyright (c) 2002 The University of Chicago, as Operator of Argonne
; National Laboratory.
; Copyright (c) 2002 The Regents of the University of California, as
; Operator of Los Alamos National Laboratory.
; This file is distributed subject to a Software License Agreement found
; in the file LICENSE that is included with this distribution. 
;*************************************************************************
PRO showMods

  ; The table below is displayed in a text widget when 'Modifications' is
  ; selected.
  ; Add  changes in table below. When necessary, change 'version'

  modString = [  $
                 $
'*************************    EDIT HISTORY    ****************************', $
'                                                                         ', $
'05/30/02  nda  Added S:VID4                                              ', $
'01/22/01  nda  Added feature to download color table to MV200            ', $
'01/22/01  nda  Changed SR:VD1/SR:VD3 to S:VID1/S:VID3 for new mvApp      ', $
'06/14/00  nda  Changed SR:VD2 to S:VID2 for new mvApp                    ', $
'12/23/98  nda  RELEASED as (Beta 2.2)                                    ', $
'12/23/98  nda  Made xxBackground from srbackground to work with selected ', $
'               system (It used to be hardcoded for SR)                   ', $
'11/24/98  nda  For all cases (including LI) poke xx:VD1:imageGrab        ', $
'               (made the Linac the same as others)                       ', $
'08/28/98  nda  Added ChoiceButton for LI/SR/SR2                          ', $
'08/13/98  nda  Swapped Variable1Name/Variable2Name in sdds file          ', $
'               so now Variable1Name = "y" and Variable2Name = "x" per MB ', $
'05/08/98  nda  RELEASED as (Beta 2.1)                                    ', $
'05/08/98  nda  If LI, then caput LI:VD1:videoRefDlyPD.GATE to 1          ', $
'05/08/98  nda  Disable WARP feature until better understood              ', $
'05/08/98  nda  Moved the printout of ID/RANGE from the image to a field  ', $
'05/08/98  nda  Allow a negative FLOOR to detect over subtraction         ', $
'05/08/98  nda  Added File menu item for srBackground Save/Restore        ', $
'05/08/98  nda  Minor cosmetic changes to lengths of text entries         ', $
'05/08/98  nda  Added EXIT after XMANAGER statement to exit IDL           ', $
'04/02/98  nda  RELEASED as (Beta 2.0)                                    ', $
'04/02/98  nda  Added "Modifications" to File Menu that prints this table ', $
'04/02/98  nda  Adjusted assignment of PVs to wfRecords to maintain odd   ', $
'               lines as odd lines, evens as evens, yet flip the image.   ', $
'               idl->ioc: 0->480, 1->479, 2->478, 3->477, ... 479->1      ', $
'04/02/98  nda  Added version variable that shows up in window title      ', $
'04/02/98  nda  Added protection against overwriting existing files       ', $
'04/02/98  nda  Made "Store Raw into Backgnd" a momentary state           ', $
'03/31/98  nda  Hitting Return in screenId increments it                  ', $
'03/31/98  nda  Changes to support new calibration file                   ', $
'03/31/98  nda  Changes to support Warping (if CAL is not a rectangle)    ', $
'               A large "W" is displayed in the lower right if warped     ', $
'03/31/98  nda  Write a 0 to xx:imageBusy upon exiting                    ', $
'03/27/98  nda  Increased value written to xx:imageBusy to 20             ', $
'03/27/98  nda  Added Odd/Even scanline choice. What about rotated images?', $
'03/27/98  nda  Print warning on profile plot if math error occurs during ', $
'               Gaussian fit                                              ', $
'03/27/98  nda  Corrected iMin/iMax for zoomed images                     ', $
'03/23/98  nda  Added "average" option for the array of 16 images         ', $
'03/13/98  nda  RELEASED as (Beta 1.0)                                    ', $
'03/13/98  nda  If imagePV begins with SR, poke SR:VD1:imageGrab          ', $
'                                                                         ', $
'                                                                         ', $
'                                                                         ']

  WT_BASE = WIDGET_BASE(GROUP_LEADER=0, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Edit History', $
      UVALUE='WT_BASE')

  WT_TEXT = WIDGET_TEXT( WT_BASE, $
     VALUE=modString, $
     UVALUE='WT_TEXT', $
     SCROLL=1, $
     XSIZE=80, $
     YSIZE=20)

  WIDGET_CONTROL, WT_BASE, /REALIZE
END


; the following procedure replots based on current view and "zoom"
PRO REPLOT

  COMMON CATCH_IMAGE, widget_ids, imageData, calInfo

  zoom = imageData.mag

  if(imageData.plotting EQ 1) THEN RETURN

  imageData.plotting = 1

  WSET, widget_ids.plot_area
  t1 = systime(1)

  ; Rotate if specified, but <for now> limit direction to < 4 so   
  ; the dimensions won't change
  dir = imageData.cal.rotation < 4
 
  ; generate 'plotImage' based on user selections
  CASE imageData.plotSrc of
      0: imageData.plotImage = imageData.rawImage 
      1: imageData.plotImage = imageData.backImage 
      2: imageData.plotImage = imageData.rawImage-imageData.backImage > $
                                                          imageData.floor 
      ELSE : imageData.plotImage = imageData.rawImage
  ENDCASE

  IF (imageData.floor GT 0) THEN BEGIN
    mask = where((imageData.plotImage EQ imageData.floor), count)
    IF(count NE 0) THEN imageData.plotImage(mask) = 0
  ENDIF
     
  ; Rotate if specified, then make sure the dimensions are corrected 
  IF(imageData.cal.rotation NE 0) THEN BEGIN
     rot = imageData.cal.rotation
     ; Most rotations flip the image dimensions. Put it back as req'd
     IF((rot EQ 2) OR (rot EQ 5) OR (rot EQ 7)) THEN BEGIN
         imageData.plotImage = ROTATE(imageData.plotImage, rot)
     ENDIF ELSE BEGIN
         ; Copy ysize X ysize image (some info lost)
         rotImage = ROTATE(imageData.plotImage, rot)
         imageData.plotImage[*,*] = 0 
         imageData.plotImage[0:imageData.ysize-1,0:imageData.ysize-1] = $
              rotImage[0:imageData.ysize-1,0:imageData.ysize-1]
     ENDELSE
  ENDIF
      
  ; Warping required ?
  IF(imageData.cal.calibrated EQ 2) THEN BEGIN
    xi = [imageData.cal.bl_x_PIX, $
          imageData.cal.tl_x_PIX, $
          imageData.cal.tr_x_PIX, $
          imageData.cal.br_x_PIX]

    yi = [imageData.cal.bl_y_PIX, $
          imageData.cal.tl_y_PIX, $
          imageData.cal.tr_y_PIX, $
          imageData.cal.br_y_PIX]

    warped_tr_x_PIX = imageData.cal.tl_x_PIX +   $
          ((imageData.cal.tr_x_PIX - imageData.cal.tl_x_PIX)/.707)
    warped_br_x_PIX = warped_tr_x_PIX

    xo = [imageData.cal.bl_x_PIX, $
          imageData.cal.tl_x_PIX, $
          warped_tr_x_PIX, $
          warped_br_x_PIX]

    warped_tr_y_PIX = imageData.cal.tl_y_PIX
    warped_br_y_PIX = imageData.cal.bl_y_PIX

    yo = [imageData.cal.bl_y_PIX, $
          imageData.cal.tl_y_PIX, $
          warped_tr_y_PIX, $
          warped_br_y_PIX]

    imageData.plotImage = WARP_TRI(xo, yo, xi, yi, imageData.plotImage)

  END


  IF (imageData.acqMode EQ 3) THEN BEGIN
     print, imageData.acqNum
     imageData.rawImages[*,*,imageData.acqNum] = imageData.rawImage
     IF imageData.acqNum EQ 0 THEN ERASE
     IF (zoom EQ 0) THEN BEGIN   ;plot full 1:1 view
         tiny = REBIN(imageData.plotImage, 64, 60)
     ENDIF ELSE BEGIN 
         ; don't know how to do this yet
         ; just plot full view for now
         tiny = REBIN(imageData.plotImage, 64, 60)
     ENDELSE
     IF(imageData.scaleINT EQ 0) THEN $
         TV, tiny, imageData.acqNum  $
     ELSE $
         TVSCL, tiny, imageData.acqNum
     imageData.acqNum = imageData.acqNum + 1
     imageData.plotting = 0
     RETURN
  ENDIF

  ERASE

  ; Size to plot window
  tplotImage = REBIN(imageData.plotImage,256,240)

  iMin = MIN(tplotImage)
  iMax = MAX(tplotImage)
  Print, 'Data Range = ', iMin, '-> ',iMax

  IF (zoom EQ 0) THEN BEGIN   ;plot full 1:1 view
    IF (imageData.view EQ 0) THEN BEGIN
      IF(imageData.scaleINT EQ 0) THEN $
          TV, tplotImage  $
      ELSE $
          TVSCL, tplotImage
      ;superimpose ROI rectangle
      IF (imageData.viewROI EQ 1) THEN BEGIN
          xmin = imageData.cal.xmin_ROI / imageData.scale
          ymin = imageData.cal.ymin_ROI / imageData.scale
          xmax = imageData.cal.xmax_ROI / imageData.scale
          ymax = imageData.cal.ymax_ROI / imageData.scale
          PLOTS, xmin, ymin, COLOR = 254, /DEVICE
          PLOTS, xmax, ymin, /CONTINUE, COLOR = 254, /DEVICE
          PLOTS, xmax, ymax, /CONTINUE, COLOR = 254, /DEVICE
          PLOTS, xmin, ymax, /CONTINUE, COLOR = 254, /DEVICE
          PLOTS, xmin, ymin, /CONTINUE, COLOR = 254, /DEVICE
      ENDIF
      IF (imageData.viewCAL EQ 1) THEN BEGIN
          xmin = imageData.cal.xmin_PIX / imageData.scale   
          ymin = imageData.cal.ymin_PIX / imageData.scale
          xmax = imageData.cal.xmax_PIX / imageData.scale
          ymax = imageData.cal.ymax_PIX / imageData.scale
          PLOTS, xmin, ymin, LineStyle=2, COLOR = 254, /DEVICE
          PLOTS, xmax, ymin, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
          PLOTS, xmax, ymax, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
          PLOTS, xmin, ymax, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
          PLOTS, xmin, ymin, /CONTINUE, LineStyle=2, COLOR = 254, /DEVICE
      ENDIF

    ENDIF ELSE BEGIN
      shade_surf, tplotImage, AZ=imageData.zRot
    ENDELSE

  ENDIF ELSE BEGIN             ;plot zoomed with user entered limits
    xmin = imageData.cal.xmin_ROI / imageData.scale
    ymin = imageData.cal.ymin_ROI / imageData.scale
    xmax = imageData.cal.xmax_ROI / imageData.scale
    ymax = imageData.cal.ymax_ROI / imageData.scale
    IF (imageData.view EQ 0) THEN BEGIN
      ; Magnify as many times as possible for display size
      mag = floor(!d.x_size/(xmax-xmin)) < $ 
            floor(!d.y_size/(ymax-ymin))
      newImage = REBIN(tplotImage(xmin:xmax,  $
                                  ymin:ymax), $
                                 (xmax-xmin+1)*mag, $
                                 (ymax-ymin+1)*mag)
      iMin = MIN(newImage)
      iMax = MAX(newImage)
      IF(imageData.scaleINT EQ 0) THEN $
          TV, newImage   $
      ELSE $
          TVSCL, newImage 
    ENDIF ELSE BEGIN
      shade_surf, tplotImage(xmin:xmax, ymin:ymax), AZ=imageData.zRot
    ENDELSE
  ENDELSE

  str1 = 'Range: ' + STRTRIM(STRING(iMin),2) + '/' + STRTRIM(STRING(iMax),2)
  str2 = STRTRIM(imageData.cal.desc,2) + '  ' + str1
  ;XYOUTS, 4,14, str2, /DEVICE, COLOR = 255
  WIDGET_CONTROL, widget_ids.image_info, SET_VALUE=str2

  IF(imageData.cal.calibrated EQ 2) THEN  $
      XYOUTS, 240, 4, 'W', /DEVICE, COLOR = 255, SIZE = 2.0

  IF(STRLEN(imageData.fName) NE 0) THEN BEGIN
      XYOUTS, 4, 4, imageData.fName, /DEVICE, COLOR = 255
  ENDIF

  ;********  Profiles **** PROFILES ******
 
  scale = imageData.scale

  roiImage =imageData.plotImage[imageData.cal.xmin_ROI:imageData.cal.xmax_ROI,$
                                imageData.cal.ymin_ROI:imageData.cal.ymax_ROI]

  x_axis = imageData.x_axisEGU[imageData.cal.xmin_ROI:imageData.cal.xmax_ROI]
  y_axis = imageData.y_axisEGU[imageData.cal.ymin_ROI:imageData.cal.ymax_ROI]

  x_profile = TOTAL(roiImage,2)
  y_profile = TOTAL(roiImage,1)

  ; Scale profiles according to MAX, which is 256 * 
  x_profile = x_profile/(N_ELEMENTS(x_profile)*256)
  y_profile = y_profile/(N_ELEMENTS(y_profile)*256)

  imageData.xp_peak = MAX(x_profile)
  imageData.yp_peak = MAX(y_profile)

  IF imageData.cal.calibrated EQ 1 THEN BEGIN
      units = imageData.cal.egu
  ENDIF ELSE BEGIN
      units = 'pixels' 
  ENDELSE

  WSET, widget_ids.x_prof_wid
  PLOT, x_axis, x_profile, TITLE = 'X Profile', XTITLE = units
  mathError = CHECK_MATH(0,1)   ;clear math error
  OPLOT, x_axis, GAUSSFIT(x_axis, x_profile, x_gaussian),LineStyle=2, THICK=2 $
         ,COLOR = 150

  mathError = CHECK_MATH(0,1)   ;clear math error
  imageData.xp_sigma  = x_gaussian[2]
  imageData.xp_center = x_gaussian[1]
  str1 = STRING(FORMAT='("Sigma = ",f7.2)',x_gaussian[2])
  str2 = STRING(FORMAT='("Center = ",f7.2)',x_gaussian[1])

  XYOUTS, 63, 207, str1, /DEVICE
  XYOUTS, 63, 197, str2, /DEVICE
  IF(mathError NE 0) THEN BEGIN
      XYOUTS, 63, 187, 'Gaussian Fit Failed', /DEVICE
  ENDIF
 
  WSET, widget_ids.y_prof_wid

  ;********* Extract Odd or Even lines, if requested 
print, N_ELEMENTS(y_profile)
  IF(imageData.lines NE 0) THEN BEGIN
      n = N_ELEMENTS(y_profile)
      indices = INDGEN(N/2) * 2
      IF(imageData.lines EQ 1) THEN BEGIN
        ;extract only odd lines
        y_profile = y_profile[indices+1]
        y_axis = y_axis[indices+1]
      ENDIF ELSE BEGIN
        ;extract only even lines
        y_profile = y_profile[indices]
        y_axis = y_axis[indices]
      ENDELSE
  ENDIF  
print, N_ELEMENTS(y_profile)

  PLOT, y_profile, y_axis, /YNOZ, TITLE = 'Y Profile' , YTITLE = units
  OPLOT, GAUSSFIT(y_axis, y_profile, y_gaussian), y_axis, LineStyle=2, THICK=2 $
         ,COLOR = 150

  mathError = CHECK_MATH(0,1)   ;clear math error
  imageData.yp_sigma  = y_gaussian[2]
  imageData.yp_center = y_gaussian[1]
  str1 = STRING(FORMAT='("Sigma = ",f7.2)',y_gaussian[2])
  str2 = STRING(FORMAT='("Center = ",f7.2)',y_gaussian[1])

  XYOUTS, 140, 207, str1, /DEVICE
  XYOUTS, 140, 197, str2, /DEVICE
  IF(mathError NE 0) THEN BEGIN
      XYOUTS, 140, 187, 'Gaussian Fit Failed', /DEVICE
  ENDIF

  WSET, widget_ids.plot_area
  ;************************

  t2 = systime(1)
  print, 'Image Plotted in ',t2-t1, ' secs'

  imageData.plotting = 0
END
;********************* End of REPLOT

PRO changeScreen, index 

  COMMON CATCH_IMAGE, widget_ids, imageData, screenCalArray

    COMMON CATCH_IMAGE, widget_ids, imageData, screenCalArray

    Print, 'Screen Change ...'

    imageData.screen_id = index
    imageData.cal = screenCalArray[index]
    WIDGET_CONTROL, widget_ids.screenID, SET_VALUE = index

    CALC_EGU_ARRAYS  ; resides in screen_cal.pro

    REPLOT
END

;**************************
;
; This function returns a 1 if the specified file exists.
;

FUNCTION FILE_EXISTS, fileName
 
  OPENR, lun, fileName, /Get_LUN, Error = err
  
  IF(err EQ 0) THEN BEGIN
      Free_LUN, lun
      RETURN, 1
  ENDIF ELSE   $
      RETURN, 0

END

;**************************

PRO filePDMENU_Event, Event

  COMMON CATCH_IMAGE, widget_ids, imageData

  P = ''

  CASE Event.Value OF 

  'File.Open (IDL) ...': BEGIN
    PRINT, 'Event for File.Open ...'
        F=PICKFILE(/READ, GET_PATH=P, FILTER = imageData.lpath + '*.idlImage')
        IF F eq '' THEN return
        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME

        IF (FILE_EXISTS(FNAME) EQ 0) THEN BEGIN
            response = WIDGET_MESSAGE('File does not exist !')
            RETURN
        ENDIF
        
        result = RSTRPOS(FNAME, '/')
        imageData.fName = STRMID(FNAME, result+1, 50)
        RESTORE, FNAME
        imageData.rawImage = saveImage
        REPLOT
    END

  'File.Save (IDL) ...': BEGIN
    PRINT, 'Event for File.Save ...'
        F=PICKFILE(/WRITE, GET_PATH=P, FILTER = imageData.lpath + '*.idlImage')
        IF F eq '' THEN return

        IF (STRMID(F,STRLEN(F)-9, 9) NE '.idlImage') THEN $
                F = F + '.idlImage'

        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME

        IF (FILE_EXISTS(FNAME) EQ 1) THEN BEGIN
            response = WIDGET_MESSAGE('Overwrite existing file ?', /QUESTION)
            IF (response EQ 'No') THEN RETURN
        ENDIF

        saveImage = imageData.plotImage
        OPENW, lun, FNAME, /Get_LUN, Error = err
        IF (err NE 0)  THEN BEGIN
            result = WIDGET_MESSAGE("FILE WRITE ERROR !")
        ENDIF ELSE BEGIN
            Free_LUN, lun
            save, saveImage, filename=FNAME
        ENDELSE
    END

  'File.Save as SDDS ...': BEGIN
    PRINT, 'Event for File.Save ...'
        F=PICKFILE(/WRITE, GET_PATH=P, FILTER = imageData.lpath + '*.sdds')
        IF F eq '' THEN return

        IF (STRMID(F,STRLEN(F)-5, 5) NE '.sdds') THEN $
                F = F + '.sdds'

        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME

        IF (FILE_EXISTS(FNAME) EQ 1) THEN BEGIN
            response = WIDGET_MESSAGE('Overwrite existing file ?', /QUESTION)
            IF (response EQ 'No') THEN RETURN
        ENDIF

        writeSDDSimage, FNAME

        convertCommand = 'sddsconvert -binary -noWarnings ' + FNAME  $
                          +';\rm ' + FNAME + '~'
        SPAWN, convertCommand, result
  
    END


  'File.Screen Cal ...': BEGIN
    PRINT, 'Event for Screen Cal ...'
    screen_cal
    END

  'File.Background Save/Restore ...': BEGIN
    PRINT, 'Event for Bkgnd ...'
    xxBackground
    END

  'File.Open Raw Image': BEGIN
    PRINT, 'Event for File.Open Raw Image'
        F=PICKFILE(/READ, GET_PATH=P, FILTER = imageData.lpath + '*.image')
        IF F eq '' THEN return
        IF (STRMID(F,0,1) EQ '/') THEN $
                FNAME = F $
        ELSE $
                FNAME = P+F

        Print, FNAME
        IF (FILE_EXISTS(FNAME) EQ 0) THEN BEGIN
            response = WIDGET_MESSAGE('File does not exist !')
            RETURN
        ENDIF
        result = RSTRPOS(FNAME, '/')
        imageData.fName = STRMID(FNAME, result+1, 50)
        saveImage = BYTARR(512,484)
        openr,1, FNAME
        readu,1, saveImage
        close,1

        imageData.rawImage = FIX(saveImage[*,0:479])
        REPLOT
    END

  'File.Quit ': BEGIN
    PRINT, 'Event for File.Quit '
    IF (STRLEN(imageData.imagePV) NE 0) THEN  BEGIN
        u=caWidgetClearmonitor(imageData.imagePV,widget_ids.imagePV_base)
        u=caWidgetClearmonitor(imageData.screenIdPv, widget_ids.screenIDBASE)
        u=caput(imageData.imagePV + 'Busy', 0)  ; clear xx:imageBusy
    ENDIF
    WIDGET_CONTROL, event.top, /DESTROY
    END

  'File.Modifications ...': BEGIN
    PRINT, 'Event for File.Help '
    showMods
    END

  ELSE:

  ENDCASE

  imageData.lpath = P

END

PRO colorPDMENU_Event, Event
  COMMON CATCH_IMAGE, widget_ids, imageData

  CASE Event.Value OF 
  'Color.Change Color Table': BEGIN
    PRINT, 'Event for Color.Change Color Table'
    XLOADCT
    END
  'Color.Send Color Table to MV200': BEGIN
    PRINT, 'Event for Color.Send Color Table'
    TVLCT, R, G, B, /GET
    PRINT, R, G, B
    IF (STRLEN(imageData.imagePV) NE 0) THEN  BEGIN
        u=caPutArray(imageData.imagePREFIX + ':setRedLutWF',  R) ; send R array
        u=caPutArray(imageData.imagePREFIX + ':setGreenLutWF',G) ; send B array
        u=caPutArray(imageData.imagePREFIX + ':setBlueLutWF', B) ; send G array
        PRINT, 'Color Table Sent'
    ENDIF
    END
  ENDCASE
END


PRO updateBGROUP_Event, Event

  COMMON CATCH_IMAGE, widget_ids, imageData

  PRINT, 'updateBGROUP_Event', Event.Value, Event.Select
  IF Event.Select EQ 1 THEN BEGIN  ; mask out 'deselect' event
      CASE Event.Value OF
        0: BEGIN
          imageData.acqMode = 1
              stat = caput(imageData.imagePV + 'Grab', 0)
          END
        1: BEGIN
          imageData.acqMode = 2
              stat = caput(imageData.imagePV + 'Grab', 0)
          END
        2: BEGIN
          imageData.acqNum  = 0
          imageData.acqMode = 3
              stat = caput(imageData.imagePV + 'Grab', 0)
          END
        3: BEGIN
          imageData.acqMode = 0
          END
      ENDCASE
   ENDIF
END

PRO subImagePDMENU_Event, Event

  COMMON CATCH_IMAGE, widget_ids, imageData

  imageData.acqMode = 0

  CASE Event.Value OF

  1: BEGIN
       PRINT, 'Event for Entire Image Array'
       FOR I = 0, imageData.imageArrayN-1 DO BEGIN
         plotImage = REBIN(imageData.rawImages(*,*,I), 64, 60)
         TV, plotImage, I
         IF(imageData.scaleINT EQ 0) THEN $
             TV, plotImage, I $
         ELSE $
             TVSCL, plotImage, I
       END
       imageData.subImageIndex = -1
     END
  18: BEGIN 
        newIndex = Event.Value - 2
        imageData.rawImage = (imageData.rawImages[*,*,0] + $
                              imageData.rawImages[*,*,1] + $
                              imageData.rawImages[*,*,2] + $
                              imageData.rawImages[*,*,3] + $
                              imageData.rawImages[*,*,4] + $
                              imageData.rawImages[*,*,5] + $
                              imageData.rawImages[*,*,6] + $
                              imageData.rawImages[*,*,7] + $
                              imageData.rawImages[*,*,8] + $
                              imageData.rawImages[*,*,9] + $
                              imageData.rawImages[*,*,10] + $
                              imageData.rawImages[*,*,11] + $
                              imageData.rawImages[*,*,12] + $
                              imageData.rawImages[*,*,13] + $
                              imageData.rawImages[*,*,14] + $
                              imageData.rawImages[*,*,15]) / 16
        imageData.subImageIndex = newIndex
        WIDGET_CONTROL, widget_ids.subImageIndex, SET_VALUE = newIndex
        REPLOT
    END
  ELSE: BEGIN
    PRINT, 'Event for ImageArray index', Event.Value
    IF ((Event.Value LT imageData.imageArrayN+2) AND $
        (Event.Value GT 1)) THEN BEGIN
        newIndex = Event.Value - 2
        imageData.rawImage = imageData.rawImages[*,*,newIndex]
        imageData.subImageIndex = newIndex
        WIDGET_CONTROL, widget_ids.subImageIndex, SET_VALUE = newIndex
        REPLOT
    END
  ENDELSE
  ENDCASE
END


;*************  Widget Event Routine  *************
;*************  WIDGET EVENT ROUTINE  *************
;
;
; Called by the XMANAGER when a widget event occurs
;
;
;**************************************************
PRO main13_2_Event, Event

  COMMON CATCH_IMAGE, widget_ids, imageData, screenCalArray

  ; First check for a few event.id's that are possible matches
  CASE event.id OF

      widget_ids.imagePV_base: $  ;channel access event from image record
        BEGIN
          stat = caget(imageData.imagePV, junk1)
          stat = caget(imageData.imagePV + 'ROI', junk1)
          Print, 'Plot Event'

          CASE imageData.acqMode OF
    
              0: RETURN    ;Don't update
 
              1: BEGIN
                   imageData.acqMode = 0    ; reset so only 1 is obtained
                   WIDGET_CONTROL, widget_ids.acq_mode, SET_VALUE = 3
                 END

              3: BEGIN
                   IF (imageData.acqNum GT 15) THEN BEGIN
                     imageData.acqMode = 0
                     WIDGET_CONTROL, widget_ids.acq_mode, SET_VALUE = 3
                     RETURN
                   ENDIF
                 END
    
              2:  ; go on through

          ENDCASE
 
          ;Set subArray index to -1 to indicate update
          imageData.subImageIndex = -1
          WIDGET_CONTROL, widget_ids.subImageIndex, SET_VALUE = -1

          IF (STRLEN(imageData.imagePV) EQ 0) THEN  RETURN
          IF ((imageData.plotting) EQ 1) THEN  RETURN

          t1 = systime(1)
          IF (imageData.fetchROI EQ 0) THEN BEGIN
              status = caGetArray(imageData.wfRecords, tempArray, $
                              max=imageData.xsize, type = imageData.psize)
              help, tempArray
              imageData.rawImage = tempArray
          ENDIF ELSE BEGIN
              firstLine = imageData.cal.ymin
              lastLine =  imageData.cal.ymax
              xstart   =  imageData.cal.xmin
              xsize    = (imageData.cal.xmax - imageData.cal.xmin)
              status = caGetArray(imageData.wfRoiRecords[firstLine:lastLine], $
                                  tempArray, $
                                  max=xsize, type = imageData.psize)
              help, tempArray
              imageData.rawImage[*,*] = 0
              imageData.rawImage[xstart:(xstart+xsize-1),  $
                                 firstLine:lastLine] = tempArray
                                 tempArray
          ENDELSE

          ; Clear filename to indicate source of image is from CA
          imageData.fName = ''

          t2 = systime(1) 
          Print, 'Partials Retrieved in ', t2-t1, ' secs'

;         s = CALL_EXTERNAL('miscRoutines.so','buildImage', $
;                           tempArray, imageData.rawImage)

          REPLOT

          ; If acqMode != 0 , schedule another grab
          IF  (imageData.acqMode NE 0) THEN                $
                  WIDGET_CONTROL, widget_ids.top_base, TIMER = 1

          RETURN
        END
  
      widget_ids.base: $  ;timer event (every 5 seconds)          
        BEGIN
          IF (STRLEN(imageData.imagePV) NE 0) THEN BEGIN
              stat = caput(imageData.imagePV + 'Busy', 20)
              WIDGET_CONTROL, widget_ids.base, TIMER = 5
          ENDIF
 
          RETURN
        END

      widget_ids.top_base: $  ;periodic poke of xx:VDx:imageGrab
        BEGIN
          IF (imageData.acqMode NE 0) THEN BEGIN
                stat = caput(imageData.imagePV + 'Grab', 0)
                WIDGET_CONTROL, widget_ids.top_base, TIMER = 1
          ENDIF
          RETURN
        END

      widget_ids.screenIDBASE: $  ;cameraSelected PV changed 
        BEGIN
          screenId = 0
          stat = caget(imageData.screenIdPv, screenId)
          IF((stat EQ 0) AND (screenId GT 0) AND (screenId LT 101)) THEN BEGIN
              changeScreen, screenId
          ENDIF ELSE BEGIN
              changeScreen, 0
          ENDELSE
          
          RETURN
        END

      ELSE:
  ENDCASE

  ; Evaluate the User_value of a widget to identify itself.

  WIDGET_CONTROL, Event.Id, GET_UVALUE=Ev

  CASE Ev OF 

  ; WID_EVENT for filePDMENU **************************************************
  'filePDMENU': filePDMENU_Event, Event

  ; WID_EVENT for color_PDMENU ************************************************
  'colorPDMENU': colorPDMENU_Event, Event


  ; WID_EVENT for screenFIELD ************************************************
  'screenFIELD': BEGIN
     newScreenId = Event.Value
     IF(newScreenId EQ imageData.screen_id) THEN $
         newScreenId = imageData.screen_id + 1
     IF ((newScreenId GT 0) AND (newScreenId LT 101)) THEN BEGIN
         changeScreen, newScreenId
     ENDIF ELSE BEGIN
         changeScreen, 0
     ENDELSE

     WIDGET_CONTROL, widget_ids.screenIDOpt, SET_VALUE = 1
     u=caWidgetClearmonitor(imageData.screenIdPv,    $
                            widget_ids.screenIDBASE)
   END

  ; WID_EVENT for screenChoiceBGROUP  ***************************************
  'screenChoiceBGROUP': BEGIN
    IF Event.Select EQ 1 THEN BEGIN  ; mask out 'deselect' event
     IF (Event.Value  EQ 0) THEN BEGIN
         screenId = 0
         stat = caget(imageData.screenIdPv, screenId)
             IF((stat EQ 0) AND (screenId GT 0) AND (screenId LT 101))THEN BEGIN
                 changeScreen, screenId
                 u=caWidgetSetmonitor(imageData.screenIdPv,    $
                                      widget_ids.screenIDBASE)
             ENDIF ELSE BEGIN
                 changeScreen, 0
                 WIDGET_CONTROL, widget_ids.screenIDOpt, SET_VALUE = 1
             ENDELSE
     ENDIF ELSE BEGIN
         WIDGET_CONTROL, widget_ids.screenID, GET_VALUE = screenId
         changeScreen, screenId
         u=caWidgetClearmonitor(imageData.screenIdPv,    $
                                widget_ids.screenIDBASE)
     ENDELSE

    ENDIF
   END

  ; WID_EVENTfor viewBGROUP  *************************************************
  'viewBGROUP': BEGIN
    IF Event.Select EQ 1 THEN BEGIN  ; mask out 'deselect' event
      CASE Event.Value OF
      0: BEGIN
         Print,'View Full Image'

         imageData.mag = 0
         REPLOT
         END

      1: BEGIN
         Print,'View ROI'

         imageData.mag = 1
         REPLOT
         END

      2: BEGIN
         Print,'Button View as image' 
      
         imageData.view = 0
         REPLOT
         END

      3: BEGIN
         Print,'Button View as a surface Pressed'
      
         imageData.view = 1
         REPLOT
         END

      ELSE: Message,'Unknown button pressed'

      ENDCASE
    ENDIF
    END

  ; WID_EVENT for displayBGROUP  *******************************************
  'displayBGROUP': BEGIN
      print,Event.Value, Event.Select
      CASE Event.Value OF
          0: BEGIN
               imageData.viewROI = Event.Select
               REPLOT
             END
          1: BEGIN
               imageData.viewCAL = Event.Select
               REPLOT
             END
          2: BEGIN
               imageData.scaleINT = Event.Select
               REPLOT
             END
          3: BEGIN
; **** Force back off until I can fully implement ROI fetch
             WIDGET_CONTROL, Event.id, GET_VALUE = current
             current[3]=0
             WIDGET_CONTROL, Event.id, SET_VALUE = current
; 
; **** Can't do this until I can force a reconnection when the ROI changes ***
;            imageData.fetchROI = Event.Select
;            IF(Event.Select EQ 0) THEN BEGIN
;              u=caWidgetClearmonitor(imageData.imagePV + 'ROI',  $
;                                     widget_ids.imagePV_base)
;              res=caWidgetSetmonitor(imageData.imagePV,          $
;                                     widget_ids.imagePV_base)
;            ENDIF ELSE BEGIN
;              u=caWidgetClearmonitor(imageData.imagePV,          $
;                                     widget_ids.imagePV_base)
;              res=caWidgetSetmonitor(imageData.imagePV + 'ROI',  $
;                                     widget_ids.imagePV_base)
;            ENDELSE
             END
          ELSE: Message,'Unknown button pressed'
          ENDCASE

    END

  ; WID_EVENT for imageBGROUP  *************************************************
  'imageBGROUP': BEGIN
      IF Event.Select EQ 1 THEN BEGIN  ; mask out 'deselect' event
          CASE Event.Value OF
          0: BEGIN
             Print,'Raw'
             imageData.plotSrc = 0
             REPLOT
             END
          1: BEGIN
             Print,'Background'
             imageData.plotSrc = 1
             REPLOT
             END
          2: BEGIN
             Print,'Raw - Background'
             imageData.plotSrc = 2
             REPLOT
             END
          3: BEGIN
             Print,'Store Raw int Background'
             imageData.backImage = imageData.rawImage
             WAIT, .5
             WIDGET_CONTROL, Event.Id, SET_VALUE = imageData.plotSrc
             END
          ELSE: Message,'Unknown button pressed'
          ENDCASE
      ENDIF
    END

  ; WID_EVENT for pvBGROUP   *************************************************
  'pvBGROUP': BEGIN
      Print, 'Event for PV Name Entry: ', Event.Select, Event.Value
      IF Event.Select EQ 0 THEN BEGIN  ; This is 'deselect' event
          ; clear monitor on existing PV if it exists
          IF (STRLEN(imageData.imagePV) NE 0) THEN  BEGIN
              u=caWidgetClearmonitor(imageData.imagePV,widget_ids.imagePV_base)
              u=caput(imageData.imagePV + 'Busy', 0)  ; clear xx:imageBusy
              imageData.imagePV = ''
              WIDGET_CONTROL, widget_ids.imagePV_stat,    $ 
                         SET_VALUE =  '>> SYSTEM CLEAR <<'
          ENDIF
          RETURN
      ENDIF
     
      IF Event.Select EQ 1 THEN BEGIN  ; mask out 'deselect' event
        CASE Event.Value OF
          0: BEGIN
                 WIDGET_CONTROL, widget_ids.imagePV_stat,    $ 
                             SET_VALUE =  '>> SELECT A SYSTEM <<'
                 RETURN
             END
          1: BEGIN
                 new_pv_name = 'LI:VD1:image'
                 new_pv_prefix = 'LI:VD1'
             END
          2: BEGIN
                 new_pv_name = 'S:VID1:image'
                 new_pv_prefix = 'S:VID1'
             END
          3: BEGIN
                 new_pv_name = 'S:VID2:image'
                 new_pv_prefix = 'S:VID2'
             END
          4: BEGIN
                 new_pv_name = 'S:VID3:image'
                 new_pv_prefix = 'S:VID3'
             END
          5: BEGIN
                 new_pv_name = 'S:VID4:image'
                 new_pv_prefix = 'S:VID4'
             END
        ENDCASE
      ENDIF

      ; Check for all needed PV's based on the one entered
      IF (STRLEN(new_pv_name) EQ 0) THEN BEGIN 
          res = -1  
      ENDIF ELSE BEGIN 
          WIDGET_CONTROL, widget_ids.imagePV_stat,    $ 
                          SET_VALUE =  'Searching for PVs'
          res = 0
          IF (caSearch(new_pv_name) NE 0) THEN res = -1
          IF (caSearch(new_pv_name + 'Busy') NE 0) THEN res = -1
          IF (caSearch(new_pv_name + 'Busy' + 'Host') NE 0) THEN res = -1
      ENDELSE
        
      IF (res EQ 0) THEN BEGIN
        ;See if image record is 'in-use' by checking BUSY PV
        res = caget(new_pv_name + 'Busy', value)
        IF((res EQ 0) and (value NE 0)) THEN BEGIN
             res = caget(new_pv_name + 'Busy' + 'Host', str0)
             str1 = 'Image PV used by : ' + str0
             result = WIDGET_MESSAGE(str1)
             WIDGET_CONTROL, widget_ids.imagePV_stat,    $ 
                             SET_VALUE =  '>> PV NOT VALID <<'
             imageData.imagePV = ''
             RETURN
        ENDIF

        WIDGET_CONTROL, widget_ids.imagePV_stat, SET_VALUE = '>> PV Valid <<', $
                        /NO_NEWLINE
        imageData.imagePV = new_pv_name
        imageData.imagePREFIX = new_pv_prefix

        ;Register hostname and start watchdog timer
        SPAWN, 'hostname', host
        SPAWN, 'whoami', user
        res = caput(imageData.imagePV + 'Busy', 20)
        res = caput(new_pv_name + 'Busy' + 'Host', user + '@' + host)
        WIDGET_CONTROL, widget_ids.base, TIMER = 5

        IF (STRMID(imageData.imagePV, 0, 2) EQ 'LI') THEN $
            res=caput('LI:VD1:videoRefDlyPD.GATE', 1)

        ; Fetch Scan lines from Bottom to Top because IDL plots from lower left
        ; In order not to swap odd lines and even lines, the following 
        ; assignments are made (Image in IOC is 512x484)
        ;
        ;    IDL Scan Line                    Scan Line in IOC
        ;            0                              480
        ;            1                              479
        ;            2                              478
        ;            3                              477
        ;            4                              476
        ;            5                              475
        ;            .                               . 
        ;            .                               . 
        ;            .                               . 
        ;           476                              4 
        ;           477                              3 
        ;           478                              2 
        ;           479                              1 
        ;
        ;
        FOR i=0, imageData.ysize-1 DO BEGIN
          imageData.wfRecords[imageData.ysize-1-i] = imageData.imagePV + $
                                     '.L' + STRTRIM(STRING(i+1),2)
          imageData.wfRoiRecords[imageData.ysize-1-i] = imageData.imagePV + $
                                     'ROI.L' + STRTRIM(STRING(i+1),2)
        ENDFOR

        res=caGet(imageData.imagePV+'.SIZE',psize)
        imageData.psize = psize+1
        imageData.acqMode = 1    ; get one image at connect time
        res=caWidgetSetmonitor(imageData.imagePV,widget_ids.imagePV_base)

      ENDIF ELSE BEGIN
        WIDGET_CONTROL, widget_ids.imagePV_stat,SET_VALUE = '>> PV NOT VALID <<'
        imageData.imagePV = ''
        result = WIDGET_MESSAGE("IMAGE PROCESS VARIABLES NOT FOUND")
      ENDELSE
    END

  ; WID_EVENT for updateBGROUP   *********************************************
  'updateBGROUP': updateBGROUP_Event, Event

  ; WID_EVENT for scanLineGROUP   *********************************************
  'scanLineBGROUP': BEGIN
      PRINT, 'scanLineBGROUP', Event.Value, Event.Select
      IF (Event.Select EQ 1) THEN BEGIN  ; mask out 'deselect' event
          imageData.lines = Event.Value
          REPLOT
      ENDIF
   END

  ; WID_EVENT for subImagePDMENU  ********************************************
  'subImagePDMENU': subImagePDMENU_Event, Event

  ; WID_EVENT for subImageFIELD   ********************************************
  'subImageFIELD': BEGIN
      IF(event.value NE imageData.subImageIndex) THEN BEGIN
          event.value = event.value + 2
      ENDIF ELSE BEGIN
          IF(event.value GE imageData.imageArrayN - 1) THEN $
              event.value = 2  $
          ELSE  $
              event.value = event.value + 3
      ENDELSE

      subImagePDMENU_Event, Event  ;call routine that updates image
   END

  ; WID_EVENT for imageDRAW      ********************************************
  'imageDRAW': BEGIN
      Print, 'Event for plot area'
      IF ((Event.PRESS EQ 1) AND (imageData.view EQ 0) $
          AND (imageData.mag EQ 0)) THEN BEGIN
        WIDGET_CONTROL, Event.Id, SENSITIVE = 0

        WSET, widget_ids.plot_area
        ; provide local variables and account for !ORDER
        x = imageData.cal.xmin_ROI / imageData.scale
        y = imageData.cal.ymin_ROI / imageData.scale
        xs = (imageData.cal.xmax_ROI - imageData.cal.xmin_ROI) / imageData.scale
        ys = (imageData.cal.ymax_ROI - imageData.cal.ymin_ROI) / imageData.scale
        Print, x,y,xs,ys
        MY_BOX_CURSOR,x,y,xs,ys,/INIT
        Print, x,y,xs,ys
        x = (x > 0) < 255
        y = (y > 0) < 255
        xs = xs > 5  
        ys = ys > 5  
        Print, x,y,xs,ys
        imageData.cal.xmin_ROI = x * imageData.scale
        imageData.cal.xmax_ROI = ((x + xs) < 255) * imageData.scale
        imageData.cal.ymin_ROI = y * imageData.scale
        imageData.cal.ymax_ROI = ((y + ys) < 239) * imageData.scale
        ; Update in cal array
        screenCalArray[imageData.screen_id].xmin_ROI = imageData.cal.xmin_ROI
        screenCalArray[imageData.screen_id].xmax_ROI = imageData.cal.xmax_ROI
        screenCalArray[imageData.screen_id].ymin_ROI = imageData.cal.ymin_ROI
        screenCalArray[imageData.screen_id].ymax_ROI = imageData.cal.ymax_ROI
        IF (STRLEN(imageData.imagePV) NE 0) THEN BEGIN
            status = caPut(imageData.imagePV + 'ROI.RXST', x )
            status = caPut(imageData.imagePV + 'ROI.RXSZ', xs)
            status = caPut(imageData.imagePV + 'ROI.RYST', y )
            status = caPut(imageData.imagePV + 'ROI.RYSZ', ys)
        ENDIF
        WAIT, .2
        WIDGET_CONTROL, Event.Id, SENSITIVE = 1
        imageData.mag = 1
        REPLOT
      ENDIF ELSE IF(Event.PRESS EQ 4) THEN BEGIN
        ;toggle zoom flag
        IF (imageData.mag EQ 1) THEN imageData.mag = 0  $
        ELSE imageData.mag = 1
         Print, 'Zoom = ', imageData.mag 
        WIDGET_CONTROL, widget_ids.plot_type, SET_VALUE = imageData.mag
        REPLOT
      ENDIF ELSE IF (Event.PRESS EQ 1) THEN BEGIN
          WSET, widget_ids.plot_area
          cursor, x, y      
          print, 'x= ',x
          print, 'y= ',y
          print, ' '
      ENDIF
      END

  ; WID_EVENT for floorFIELD     ********************************************
  'floorFIELD': BEGIN
      Print, 'Event for floor entry'
      imageData.floor = Event.Value
      REPLOT
      END

  ; WID_EVENT for zRotFIELD      ********************************************
  'zRotFIELD': BEGIN
      Print, 'Event for zRot entry'
      imageData.zRot = Event.Value
      REPLOT
      END

  ; WID_EVENT for commentTEXT    ********************************************
  'commentTEXT': BEGIN
      Print, 'Event for Information Block'
      END

  ; WID_EVENT for fileTEXT       ********************************************
  'fileTEXT': BEGIN
      Print, 'Event for Snap File Name'
      END
  ELSE:     ;don't stop of no matches
  ENDCASE
END

;********************** Main Program ********************
PRO catchImage, xsize, ysize,imNum, GROUP=Group

  version = 'Beta 3.0'
  device, pseudo=8

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }


  main13_2 = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Image Capture  (' + version + ')', $
      UVALUE='main13_2')

  BASE68 = WIDGET_BASE(main13_2, $
      COLUMN=4, $
      FRAME=2, $
      MAP=1, $
      TITLE='Top Menu Line', $
      UVALUE='BASE68')

  BASE190 = WIDGET_BASE(BASE68, $
      COLUMN=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='menu base', $
      UVALUE='BASE190')

  fileMenuDesc = [ $
      { CW_PDMENU_S,       3, 'File' }, $ ;        0
        { CW_PDMENU_S,       0, 'Open Raw Image' }, $ ;        2
        { CW_PDMENU_S,       0, 'Open (IDL) ...' }, $ ;        1
        { CW_PDMENU_S,       0, 'Save (IDL) ...' }, $ ;        2
        { CW_PDMENU_S,       0, 'Save as SDDS ...' }, $ ;        2
        { CW_PDMENU_S,       0, 'Screen Cal ...' }, $ ;        2
        { CW_PDMENU_S,       0, 'Background Save/Restore ...' }, $ ;        2
        { CW_PDMENU_S,       0, ' '},               $ ;        2
        { CW_PDMENU_S,       0, 'Modifications ...'},  $ ;        2
        { CW_PDMENU_S,       0, ' '},               $ ;        2
        { CW_PDMENU_S,       2, 'Quit ' } $  ;      3
  ]
  filePDMENU = CW_PDMENU( BASE190, fileMenuDesc, /RETURN_FULL_NAME, $
      UVALUE='filePDMENU')

  BASE191 = WIDGET_BASE(BASE68, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='ScreenID', $
      UVALUE='BASE191')

  screenChoiceDesc = [ $
    'Get Screen ID from EPICS', $
    'Get Screen ID from USER'  ]
  screenChoiceBGROUP = CW_BGROUP( BASE191, screenChoiceDesc, $
      ROW=2, $
      EXCLUSIVE=1, $
      SET_VALUE = 0,  $
      UVALUE='screenChoiceBGROUP')

  screenFIELD = CW_FIELD( BASE191,VALUE='0', $
      INTEGER=1, $
      RETURN_EVENTS=1, $
      TITLE='Screen ID #', $
      UVALUE='screenFIELD', $
      XSIZE=6)

  colorMenuDesc = [ $
      { CW_PDMENU_S,       3, 'Color' }, $ ;        0
        { CW_PDMENU_S,       0, 'Change Color Table' }, $  ;      1
        { CW_PDMENU_S,       2, 'Send Color Table to MV200' } $  ;      2
  ]
  colorPDMENU = CW_PDMENU( BASE190, colorMenuDesc, /RETURN_FULL_NAME, $
      UVALUE='colorPDMENU')


  BASE177 = WIDGET_BASE(BASE68, $
      ROW=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='image/surf select base', $
      UVALUE='BASE177')

  viewButDesc = [ $
    'View entire image', $
    'View ROI', $
    'View as image', $
    'View as a surface' ]
  viewBGROUP = CW_BGROUP( BASE177, viewButDesc, $
      ROW=4, $
      EXCLUSIVE=1, $
      UVALUE='viewBGROUP')

  BASE185 = WIDGET_BASE(BASE68, $
      ROW=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='unit select base', $
      UVALUE='BASE185')

  imageButDesc = [ $
    'Raw Image', $
    'Background', $
    'Raw - Background', $
    'Store Raw into Backgnd' ]

  imageBGROUP = CW_BGROUP( BASE185, imageButDesc, $
      ROW=4, $
      EXCLUSIVE=1, $
      UVALUE='imageBGROUP')

  BASE221 = WIDGET_BASE(main13_2, $
      COLUMN=4, $
      MAP=1, $
      TITLE='PV Name(s) Base', $
      UVALUE='BASE221')

  BASE194 = WIDGET_BASE(BASE221, $
      Column=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='PV Prefix', $
      UVALUE='BASE194')

; LABEL215 = WIDGET_LABEL( BASE194, $
;     UVALUE='LABEL215')

; pvFIELD = CW_FIELD( BASE194,VALUE='', $
;     ROW=1, $
;     STRING=1, $
;     RETURN_EVENTS=1, $
;     TITLE=' ', $
;     UVALUE='pvFIELD', $
;     XSIZE=24)

  pvButDesc = [ $
    'NONE', $
    'LINAC (LI:VD1)', $
    'MCR MV200 (S:VID1)', $
    'S35A (S:VID2)', $
    'S35B (S:VID3)', $
    'S35C (S:VID4)' ]

  pvBGROUP = CW_BGROUP( BASE194, pvButDesc, $
      ROW=6, $
      EXCLUSIVE=1, $
      UVALUE='pvBGROUP')

  LABEL217 = WIDGET_TEXT( BASE194, $
      UVALUE='LABEL217', $
      VALUE='>> Select a System <<', $
      XSIZE=24, $
      YSIZE=1)

  BASE208 = WIDGET_BASE(BASE221, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Options', $
      UVALUE='BASE208')

  updateMenuDesc = [ $
        'Update Image Once' , $ 
        'Update Image Cont' , $
        'Acquire 16' , $ 
        'Stop Acq' ]
  updateBGROUP = CW_BGROUP( BASE208, updateMenuDesc, $
      ROW=4, $
      SET_VALUE = 3,  $
      EXCLUSIVE = 1,  $
      UVALUE='updateBGROUP')

  BASE1000 = WIDGET_BASE(BASE208, $
      Column=2, $
      FRAME=2, $
      MAP=1, $
      TITLE='Misc Menus', $
      UVALUE='BASE1000')

  subImageMenuDesc = [ $
      { CW_PDMENU_S,       3, 'Image Array' }, $
        { CW_PDMENU_S,       0, 'All 16' }, $
        { CW_PDMENU_S,       0, 'Image 0' }, $
        { CW_PDMENU_S,       0, 'Image 1' }, $
        { CW_PDMENU_S,       0, 'Image 2' }, $
        { CW_PDMENU_S,       0, 'Image 3' }, $
        { CW_PDMENU_S,       0, 'Image 4' }, $
        { CW_PDMENU_S,       0, 'Image 5' }, $
        { CW_PDMENU_S,       0, 'Image 6' }, $
        { CW_PDMENU_S,       0, 'Image 7' }, $
        { CW_PDMENU_S,       0, 'Image 8' }, $
        { CW_PDMENU_S,       0, 'Image 9' }, $
        { CW_PDMENU_S,       0, 'Image 10' }, $
        { CW_PDMENU_S,       0, 'Image 11' }, $
        { CW_PDMENU_S,       0, 'Image 12' }, $
        { CW_PDMENU_S,       0, 'Image 13' }, $
        { CW_PDMENU_S,       0, 'Image 14' }, $
        { CW_PDMENU_S,       0, 'Image 15' }, $
        { CW_PDMENU_S,       2, 'Average' } $
  ]
  subImagePDMENU = CW_PDMENU( BASE1000, subImageMenuDesc, $
      UVALUE='subImagePDMENU')

  subImageFIELD = CW_FIELD( BASE1000, $
      VALUE='0', $
      ROW=1, $
      INTEGER=1, $
      RETURN_EVENTS=1, $
      TITLE = 'Index', $
      UVALUE='subImageFIELD', $
      XSIZE=3)


  BASE209 = WIDGET_BASE(BASE221, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Options', $
      UVALUE='BASE209')

  displayButDesc = [ $
    'Show ROI Box', $
    'Show Cal Box', $
    'Scale Intensity', $
    'Fetch ROI Only' ]
  displayBGROUP = CW_BGROUP( BASE209, displayButDesc, $
      ROW=5, $
      NONEXCLUSIVE=1, $
      SET_VALUE = 1,  $
      UVALUE='displayBGROUP')

  BASE210 = WIDGET_BASE(BASE221, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Options', $
      UVALUE='BASE210')

  LABEL216 = WIDGET_LABEL( BASE210, $
      UVALUE='LABEL216', $
      VALUE='Profile ...')
  scanLineDesc = [ $
    'ALL Lines (0)', $
    'ODD Lines Only (1)', $
    'EVEN Lines Only (2)' ]
  scanLineBGROUP = CW_BGROUP( BASE210, scanLineDesc, $
      ROW=3, $
      EXCLUSIVE = 1, $
      SET_VALUE = 0,  $
      UVALUE='scanLineBGROUP')

  BASE59 = WIDGET_BASE(main13_2, $
      COLUMN=1, $
      MAP=1, $
      TITLE='Plot Area', $
      UVALUE='BASE59')

  LABEL60 = WIDGET_LABEL( BASE59, $
      VALUE='Image Data (LMB - ROI Box : RMB - Toggles View)')

  LABEL61 = WIDGET_LABEL( BASE59, $
      VALUE='ROI Box: LMB-Move, MMB-Resize, RMB-Enter')

  BASE62 = WIDGET_BASE(main13_2, $
      COLUMN=3, $
      MAP=1, $
      TITLE='Plot Area', $
      UVALUE='BASE62')

  imageDRAW = WIDGET_DRAW( BASE62, $
      BUTTON_EVENTS=1, $
      RETAIN=2, $
      UVALUE='imageDRAW', $
      XSIZE=256, $
      YSIZE=240)

  xProDRAW = WIDGET_DRAW( BASE62, $
      BUTTON_EVENTS=1, $
      RETAIN=2, $
      UVALUE='xProDRAW', $
      XSIZE=256, $
      YSIZE=240)

  yProDRAW = WIDGET_DRAW( BASE62, $
      BUTTON_EVENTS=1, $
      RETAIN=2, $
      UVALUE='yProDRAW', $
      XSIZE=256, $
      YSIZE=240)


  BASE151 = WIDGET_BASE(main13_2, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='User Values', $
      UVALUE='BASE151')

  LABEL152 = WIDGET_LABEL( BASE151, $
      UVALUE='LABEL152', $
      VALUE='Display Options')

  BASE153 = WIDGET_BASE(BASE151, $
      COLUMN=4, $
      MAP=1, $
      TITLE='user entered fields', $
      UVALUE='BASE153')

  rangeTEXT = WIDGET_TEXT( BASE153, $
      VALUE='', $
      UVALUE='rangeTEXT', $
      XSIZE=40)

  floorFIELD = CW_FIELD( BASE153, $
      VALUE='0', $
      ROW=1, $
      INTEGER=1, $
      RETURN_EVENTS=1, $
      TITLE='Floor', $
      UVALUE='floorFIELD', $
      XSIZE=6)

  zRotFIELD = CW_FIELD( BASE153, $
      VALUE='30.0', $
      ROW=1, $
      FLOAT=1, $
      RETURN_EVENTS=1, $
      TITLE='Z Rotation', $
      UVALUE='zRotFIELD', $
      XSIZE=6)

  BASE129 = WIDGET_BASE(main13_2, $
      COLUMN=1, $
      FRAME=2, $
      MAP=1, $
      TITLE='Info Block', $
      UVALUE='BASE129')

  LABEL131 = WIDGET_LABEL( BASE129, $
      UVALUE='LABEL131', $
      VALUE='Comment Line')

  commentTEXT = WIDGET_TEXT( BASE129,VALUE='', $
      EDITABLE=1, $
      UVALUE='commentTEXT', $
      XSIZE=80, $
      YSIZE=1)

  LABEL135 = WIDGET_LABEL( BASE129, $
      UVALUE='LABEL135', $
      VALUE='Associated Snapshot File Name')

  fileTEXT = WIDGET_TEXT( BASE129,VALUE='', $
      EDITABLE=1, $
      UVALUE='fileTEXT', $
      XSIZE=80, $
      YSIZE=1)


  WIDGET_CONTROL, main13_2, /REALIZE

  ; Get drawable window index

  COMMON imageDRAW_Comm, imageDRAW_Id
  COMMON xProDRAW_Comm, xProDRAW_Id
  COMMON yProDRAW_Comm, yProDRAW_Id
  WIDGET_CONTROL, imageDRAW, GET_VALUE=imageDRAW_Id
  WIDGET_CONTROL, xProDRAW,  GET_VALUE=xProDRAW_Id
  WIDGET_CONTROL, yProDRAW,  GET_VALUE=yProDRAW_Id

@catchImage.init

  OPENR, lun, imageData.calFile, Error = err, /get_lun

  IF(err NE 0) THEN BEGIN
      result = WIDGET_MESSAGE("NO CALIBRATION FILE FOUND")
  ENDIF ELSE BEGIN
      ; PrintF, lun, screenCalArray did not separate structures with CR's
      ; so I guess I'm stuck with formatting it myself  :(

      desc = '          '
      vers = 0L
      ReadF, lun, desc, vers, FORMAT = '((A10), (I))'
      print, desc, vers

      IF(vers EQ imageData.calFileVer) THEN BEGIN
          FOR I = 0, 100 DO BEGIN
              ReadF, lun, calInfo, FORMAT = imageData.calFormat
              screenCalArray[I] = calInfo
              imageData.cal = screenCalArray[imageData.screen_id]
          END
          REPLOT
      ENDIF ELSE BEGIN
          result = WIDGET_MESSAGE("CALIBRATION FILE VERSION ERROR")
      ENDELSE

      Free_LUN, lun
  END

  XMANAGER, 'main13_2', MAIN13_2
  EXIT
END
